@@ -2,14 +2,33 @@
 ##
 ## in annotation L&W AGM == Liu & West "A General Algorithm"
 ## 
-## params = the initial particles for the parameter values
-##          these are drawn from the prior distribution for the parameters
-## est = names of parameters to estimate.  Other parameters are not updated.
+## params = the initial particles for the parameter values;
+##          these should be drawn from the prior distribution for the parameters
+## est = names of parameters to estimate; other parameters are not updated.
 ## smooth = parameter 'h' from AGM
-## ntries = number of samplesto draw from x_t+1|x(k)_t to estimate mean of mu(k)_t+1 as in sect 2.2 Liu & West
+## ntries = number of samplesto draw from x_{t+1} | x(k)_{t} to estimate
+##          mean of mu(k)_t+1 as in sect 2.2 Liu & West
 ## lower  = lower bounds on prior
 ## upper  = upper bounds on prior
 
+setClass(
+         "bsmcd.pomp",
+         contains="pomp",
+         representation=representation(
+           transform="logical",
+           post="array",
+           prior="array",
+           est="character",
+           eff.sample.size="numeric",
+           smooth="numeric",
+           seed="integer",
+           nfail="integer",
+           cond.log.evidence="numeric",
+           log.evidence="numeric",
+           weights="numeric"
+           )
+         )
+
 setGeneric("bsmc",function(object,...)standardGeneric("bsmc"))
 
 setMethod(
@@ -23,14 +42,15 @@ setMethod(
                     seed = NULL,
                     verbose = getOption("verbose"),
                     max.fail = 0,
+                    transform = FALSE,
                     ...) {
 
+            transform <- as.logical(transform)
+
             if (missing(seed)) seed <- NULL
             if (!is.null(seed)) {
-              if (!exists(".Random.seed",where=.GlobalEnv)) {
-                ## need to initialize the RNG
-                runif(1)
-              }
+              if (!exists(".Random.seed",where=.GlobalEnv))
+                runif(1) ## need to initialize the RNG
               save.seed <- get(".Random.seed",pos=.GlobalEnv)
               set.seed(seed)
             }
@@ -38,8 +58,8 @@ setMethod(
             error.prefix <- paste(sQuote("bsmc"),"error: ")
 
             if (missing(params)) {
-              if (length(object@params)>0) {
-                params <- object@params
+              if (length(coef(object))>0) {
+                params <- coef(object)
               } else {
                 stop(error.prefix,sQuote("params")," must be supplied",call.=FALSE)
               }
@@ -47,8 +67,11 @@ setMethod(
 
             if (missing(Np)) Np <- NCOL(params)
             else if (is.matrix(params)&&(Np!=ncol(params)))
-              stop(error.prefix,sQuote("Np")," cannot be other than ",sQuote("ncol(params)"),call.=FALSE)
+              warning(sQuote("Np")," is ignored when ",sQuote("params")," is a matrix")
             
+            if (transform)
+              params <- partrans(object,params,dir="inverse")
+
             ntimes <- length(time(object))
             if (is.null(dim(params))) {
               params <- matrix(
@@ -61,10 +84,10 @@ setMethod(
                                  )
                                )
             }
-            prior <- params
 
             npars <- nrow(params)
             paramnames <- rownames(params)
+            prior <- params
 
             if (missing(est))
               est <- paramnames[apply(params,1,function(x)diff(range(x))>0)]
@@ -109,14 +132,21 @@ setMethod(
               }
             }
 
-            xstart <- init.state(object,params=params)
+            xstart <- init.state(
+                                 object,
+                                 params=if (transform) {
+                                   partrans(object,params,dir="forward")
+                                 } else {
+                                   params
+                                 }
+                                 )
             statenames <- rownames(xstart)
             nvars <- nrow(xstart)
             
             times <- time(object,t0=TRUE)
             x <- xstart
 
-            loglik <- rep(NA,ntimes)
+            evidence <- rep(NA,ntimes)
             eff.sample.size <- rep(NA,ntimes)
             nfail <- 0
             
@@ -142,17 +172,18 @@ setMethod(
                       )
               }
 
-              X <- matrix(data=x,nrow=nvars,ncol=Np*ntries)
-              rownames(X) <- statenames
-              P <- matrix(data=params,nrow=npars,ncol=Np*ntries)
-              rownames(P) <- paramnames
               ## update mean of states at time nt as per L&W AGM (1) 
               tries <- rprocess(
                                 object,
-                                xstart=X,
+                                xstart=parmat(x,nrep=ntries),
                                 times=times[c(nt,nt+1)],
-                                params=P
-                                )[,,2,drop=FALSE]
+                                params=if (transform) {
+                                  partrans(object,params,dir="forward")
+                                } else {
+                                  params
+                                },
+                                offset=1
+                                )
               dim(tries) <- c(nvars,Np,ntries,1)
               mu <- apply(tries,c(1,2,4),mean)
               rownames(mu) <- statenames
@@ -165,8 +196,13 @@ setMethod(
                             y=object@data[,nt,drop=FALSE],
                             x=mu,
                             times=times[nt+1],
-                            params=m											
+                            params=if (transform) {
+                              partrans(object,m,dir="forward")
+                            } else {
+                              m
+                            }
                             )	
+              storeForEvidence1 <- log(sum(g))
               ## sample indices -- From L&W AGM (2)
 ##              k <- .Call(systematic_resampling,g)
               k <- sample.int(n=Np,size=Np,replace=TRUE,prob=g)
@@ -180,7 +216,7 @@ setMethod(
                                            n=Np,
                                            mean=rep(0,npars.est),
                                            sigma=hsq*params.var,
-                                           method="eigen"
+                                           method="svd"
                                            ),
                           silent=FALSE
                           )
@@ -190,13 +226,21 @@ setMethod(
                 stop(error.prefix,"extreme particle depletion",call.=FALSE)
               params[estind,] <- m[estind,]+t(pvec)
 
+              if (transform)
+                tparams <- partrans(object,params,dir="forward")
+              
               ## sample current state vector x^(g)_(t+1) as per L&W AGM (4)
               X <- rprocess(
                             object,
                             xstart=x[,k,drop=FALSE],
                             times=times[c(nt,nt+1)],
-                            params=params
-                            )[,,2,drop=FALSE]
+                            params=if (transform) {
+                              tparams
+                            } else {
+                              params
+                            },
+                            offset=1
+                            )
 
               ## evaluate likelihood of observation given X (from L&W AGM (4))
               numer <- dmeasure(
@@ -204,10 +248,16 @@ setMethod(
                                 y=object@data[,nt,drop=FALSE],
                                 x=X,
                                 times=times[nt+1],
-                                params=params
+                                params=if (transform) {
+                                  tparams
+                                } else {
+                                  params
+                                }
                                 )
               ## evaluate weights as per L&W AGM (5)
-              weights <- numer/g
+
+	      weights <- numer/g
+	      storeForEvidence2 <- log(mean(weights))
               
               ## apply box constraints as per the priors          
               for (j in seq_len(Np)) {
@@ -248,12 +298,12 @@ setMethod(
                 nfail <- nfail+1
                 if (nfail > max.fail)
                   stop(error.prefix,"too many filtering failures",call.=FALSE)
-                loglik[nt] <- log(tol)          # worst log-likelihood
+                evidence[nt] <- log(tol)          # worst log-likelihood
                 weights <- rep(1/Np,Np)
                 eff.sample.size[nt] <- 0
               } else {                  # not all particles are lost
                 ## compute log-likelihood
-                loglik[nt] <- log(mean(weights))
+                evidence[nt] <- storeForEvidence1+storeForEvidence2
                 weights[failures] <- 0
                 weights <- weights/sum(weights)
                 ## compute effective sample-size
@@ -279,16 +329,67 @@ setMethod(
               seed <- save.seed
             }
             
-            list(
-                 post=params,
-                 prior=prior,
-                 eff.sample.size=eff.sample.size,
-                 cond.loglik=loglik,
-                 smooth=smooth,
-                 seed=seed,
-                 nfail=nfail,
-                 loglik=sum(loglik),
-                 weights=weights
-                 )
+            ## if (transform) {
+            ##   params <- partrans(object,params,dir="forward")
+            ##   prior <- partrans(object,prior,dir="forward")
+            ## }
+
+            ## replace parameters with point estimate (posterior mean)
+            coef(object,transform=transform) <- apply(params,1,mean)
+
+            new(
+                "bsmcd.pomp",
+                object,
+                transform=transform,
+                post=params,
+                prior=prior,
+                est=as.character(est),
+                eff.sample.size=eff.sample.size,
+                smooth=smooth,
+                seed=as.integer(seed),
+                nfail=as.integer(nfail),
+                cond.log.evidence=evidence,
+                log.evidence=sum(evidence),
+                weights=weights
+                )
+          }
+          )
+
+setMethod("$",signature(x="bsmcd.pomp"),function (x,name) slot(x,name))
+
+bsmc.plot <- function (prior, post, est, breaks, thin, ...) {
+  prior <- t(prior[est,sample.int(n=ncol(prior),size=min(thin,ncol(prior)))])
+  post <- t(post[est,sample.int(n=ncol(post),size=min(thin,ncol(post)))])
+  all <- rbind(prior,post)
+  pairs(
+        all,
+        labels=est,
+        panel=function (x, y, ...) { ## prior, posterior pairwise scatterplot
+          op <- par(new=TRUE)
+          on.exit(par(op))
+          i <- which(x[1]==all[1,])
+          j <- which(y[1]==all[1,])
+          points(x,y,pch=20,col=rgb(0.85,0.85,0.85,0.1),xlim=range(all[,i]),ylim=range(all[,j]))
+          points(post[,i],post[,j],pch=20,col=rgb(0,0,1,0.01))
+        },
+        diag.panel=function (x, ...) { ## marginal posterior histogram
+          i <- which(x[1]==all[1,])
+          bks <- hist(c(post[,i],prior[,i]),breaks=breaks,plot=FALSE)$breaks
+          y1 <- hist(post[,i],breaks=bks,plot=FALSE)$counts
+          usr <- par('usr')
+          op <- par(usr=c(usr[1:2],0,1.5*max(y1)))
+          on.exit(par(op))
+          rect(head(bks,-1),0,tail(bks,-1),y1,col=rgb(0,0,1,1),border=NA,...)
+        }
+        )
+}
+
+setMethod(
+          "plot",
+          signature(x="bsmcd.pomp"),
+          function (x, ..., breaks, thin) {
+            if (missing(thin)) thin <- Inf
+            if (missing(breaks)) breaks <- 30
+            bsmc.plot(prior=x@prior,post=x@post,est=x@est,breaks=breaks,thin=thin,...)
           }
           )
